<?php
namespace Mnet\Admin;

use Mnet\MnetDbManager;
use Mnet\Admin\MnetPluginUtils;
use Mnet\Admin\MnetLogManager;
use Mnet\Admin\MnetAdTag;
use Mnet\Admin\MnetModuleManager;
use Mnet\Utils\DefaultOptions;
use Mnet\Utils\MnetAdSlot;

class MnetAuthManager
{
    public static $ACCESS_TOKEN_KEY = 'MNET_ACCESS_TOKEN';
    public static $CRID_KEY = 'MNET_CRID';
    public static $WP_OPTION_CUSTOMER_NAME = 'MNET_CUSTOMER_NAME';
    public static $WP_OPTION_CUSTOMER_EMAIL = 'MNET_CUSTOMER_EMAIL';
    public static $WP_OPTION_CUSTOMER_INACTIVE = 'MNET_CUSTOMER_INACTIVE';
    public static $WP_OPTION_EAP_CUSTOMER = 'MNET_EAP_CUSTOMER';
    public static $WP_OPTION_MNET_SITE_REJECT = 'MNET_SITE_REJECT';
    public static $WP_OPTION_MNET_SITE_MAPPED = 'MNET_SITE_MAPPED';
    public static $WP_OPTION_MNET_SITE_STATUS = 'MNET_SITE_STATUS';


    public static function authenticateUser()
    {
        if (!isset($_POST['email']) || empty($_POST['email'])) {
            \wp_send_json_error('Email is required', 422);
        }

        if (!isset($_POST['password']) || empty($_POST['password'])) {
            \wp_send_json_error('Password is required', 422);
        }
        try {
            $email = $_POST['email'];
            $password = $_POST['password'];
            $query = $_POST['query'];

            $fp = fopen(__DIR__ . "/../wp_public.key", "r");
            $pub_key = fread($fp, 8192);
            fclose($fp);
            openssl_get_publickey($pub_key);
            openssl_public_encrypt($email, $crypt_email, $pub_key);
            openssl_public_encrypt($password, $crypt_password, $pub_key);
            $response = self::callAuthApi(base64_encode($crypt_email), base64_encode($crypt_password), $query);
            if ($response['status'] != 'success') {
                \wp_send_json_error(
                    array(
                        'message' => isset($response['message']) ? $response['message'] : 'Error occurred. Please try again.',
                        'info' =>  isset($response['info']) ? $response['info'] : null
                    ),
                    400
                );
            }
            // Response::success($response);
            $refreshAdtags = self::setUserWpOptions($email, $response);
            self::updateAdtags($$response, $refreshAdtags);
            self::generateLoginResponse($response);
        } catch (\Exception $e) {
            self::logError($email, $e->getMessage());
            \wp_send_json_error(
                $e->getMessage(),
                500
            );
        }
    }

    private static function logError($email, $message)
    {
        $details = array_merge(
            array(
                'email' => $email,
                'domain' => MnetPluginUtils::getDomain(),
                'error' => $message
            ),
            MnetPluginUtils::getServerInfo(),
            MnetPluginUtils::getClientInfo()
        );
        MnetLogManager::logEvent('Login Error', json_encode($details));
    }

    public static function generateLoginResponse($apiResponse)
    {
        setcookie('mnet_authenticated', 1, strtotime('+7 days'), '/');
        \wp_send_json(array(
            'status' => 'SUCCESS',
            'message' => 'AUTHENTICATED',
            'slotCount' => MnetAdSlot::count(),
            'adtagCount' => MnetAdTag::count(),
            'siteRejected' => \Arr::get($apiResponse, 'account_info.eapRejected'),
            'isEap' => \Arr::get($apiResponse, 'account_info.isEap'),
            'siteStatus' => self::getSiteStatus(),
            'availableAdSizes' => MnetAdTag::getAvailableSizes(),
            'modules' => MnetModuleManager::getModules($apiResponse),
            'adstxtParserPath' => \plugin_dir_url(__DIR__) . mnet_normalize_chunks('/../../dist/js/mnetAdstxtParseWorker.js'),
        ));
    }

    public static function callAuthApi($email, $password, $query)
    {
        $url = MNET_API_ENDPOINT . "login";
        $payload = array_merge(array(
            'email' => $email,
            'password' => $password,
            'query' => $query,
            'domain' => MnetPluginUtils::getDomain(),
            'httpHost' => $_SERVER['HTTP_HOST'],
        ), MnetPluginUtils::getClientInfo());
        $response = \wp_remote_post(
            $url,
            array_merge(
                DefaultOptions::$MNET_API_DEFAULT_ARGS,
                array(
                    'method' => 'POST',
                    'body' => $payload,
                )
            )
        );
        if (\is_wp_error($response)) {
            self::logError($email, $response->get_error_message());
            if (substr($response->get_error_message(), 0, 13) === "cURL error 28") { // timeout
                return array('status' => 'fail', 'message' => 'Server timeout. Please try again.');
            }
            return array('status' => 'fail');
        }

        $body = json_decode($response['body'], true);
        return $body;
    }

    public static function updateAdtags($response, $refreshAdtags = false)
    {
        if (\Arr::get($response, 'adtags') !== null) {
            MnetAdTag::populateAdtags(\Arr::get($response, 'adtags'), \Arr::get($response, 'adHeadCodes'));
        } elseif ($refreshAdtags) {
            MnetAdTag::fetchAdTags($response['access_token'], false);
        }
    }

    public static function getCustomerName()
    {
        return \get_option(self::$WP_OPTION_CUSTOMER_NAME, 'User');
    }

    public static function getCustomerEmail()
    {
        return \get_option(self::$WP_OPTION_CUSTOMER_EMAIL, '');
    }

    public static function getCustomerId()
    {
        return \get_option(self::$CRID_KEY);
    }

    public static function getIsEapCustomer()
    {
        return \get_option(self::$WP_OPTION_EAP_CUSTOMER, 0);
    }

    public static function isSiteReject()
    {
        return \get_option(self::$WP_OPTION_MNET_SITE_REJECT, 0);
    }

    public static function getSiteMapped($data = null)
    {
        if (!is_null($data)) return intval(\Arr::get($data, 'account_info.siteMapped', 0));
        return \get_option(self::$WP_OPTION_MNET_SITE_MAPPED, false);
    }

    public static function getIsCustomerInactive($data = null)
    {
        if (!is_null($data)) return intval(\Arr::get($data, 'account_info.inactive', 0));
        return \get_option(self::$WP_OPTION_CUSTOMER_INACTIVE, false);
    }

    public static function getSiteStatus()
    {
        return \get_option(self::$WP_OPTION_MNET_SITE_STATUS, '');
    }

    public static function getAccessToken()
    {
        return \get_option(self::$ACCESS_TOKEN_KEY);
        // return MnetSessionManager::get(self::ACCESS_TOKEN_KEY);
    }

    public static function isLoggedIn()
    {
        return !!self::getAccessToken();
    }

    public static function returnIfSessionExpired()
    {
        if (!self::getAccessToken()) {
            self::clearLoggedInUserOptions();
            \wp_send_json(array('status' => MNET_SESSION_STATUS_EXPIRED), 401);
        }
    }

    public static function handleAccessTokenExpired($response)
    {
        if (isset($response['response']) && isset($response['response']['code']) && $response['response']['code'] === 401) {
            self::clearLoggedInUserOptions();
            \wp_send_json(array('status' => MNET_SESSION_STATUS_EXPIRED), 401);
        }
    }

    public static function clearExpiredToken($response)
    {
        if (!\is_wp_error($response) && isset($response['response']) && isset($response['response']['code']) && $response['response']['code'] === 401) {
            self::clearLoggedInUserOptions();
        }
    }

    public static function removeAuthCookie()
    {
        unset($_COOKIE['mnet_authenticated']);
        setcookie('mnet_authenticated', null, -1, '/');
    }

    public static function setUserWpOptions($email, $response)
    {
        // set access token
        MnetPluginUtils::setWpOption(self::$ACCESS_TOKEN_KEY, $response['access_token']);

        $name = $response['account_info']['name'];
        $name = empty($name) ? 'User' : $name;

        $newId = $response['account_info']['cid'];
        $oldId = self::getCustomerId();
        $refreshAdtags = $oldId !== $newId;

        MnetPluginUtils::setWpOption(self::$CRID_KEY, $newId);
        MnetPluginUtils::setWpOption(self::$WP_OPTION_CUSTOMER_EMAIL, $email);
        MnetPluginUtils::setWpOption(self::$WP_OPTION_CUSTOMER_NAME, $name);
        MnetPluginUtils::setWpOption(self::$WP_OPTION_EAP_CUSTOMER, intval(\Arr::get($response, 'account_info.isEap', 0)));
        MnetPluginUtils::setWpOption(self::$WP_OPTION_MNET_SITE_REJECT, intval(\Arr::get($response, 'account_info.eapRejected', 0)));
        MnetPluginUtils::setWpOption(self::$WP_OPTION_MNET_SITE_MAPPED, intval(\Arr::get($response, 'account_info.siteMapped', 0)));
        MnetPluginUtils::setWpOption(self::$WP_OPTION_CUSTOMER_INACTIVE, intval(\Arr::get($response, 'account_info.inactive', 0)));
        MnetPluginUtils::setWpOption(self::$WP_OPTION_MNET_SITE_STATUS, \Arr::get($response, 'account_info.siteStatus', ''));

        return $refreshAdtags;
    }

    public static function clearUserOptions()
    {
        \delete_option(self::$WP_OPTION_CUSTOMER_EMAIL);
        \delete_option(MnetDbManager::$WP_OPTION_MNET_DB_VERSION);
        \delete_option(self::$CRID_KEY);
        \delete_option(MnetAdTag::$AD_HEAD_CODE_KEY);
        self::clearLoggedInUserOptions();
    }

    public static function clearLoggedInUserOptions()
    {
        static::removeAuthCookie();
        \delete_option(self::$ACCESS_TOKEN_KEY);
        \delete_option(self::$WP_OPTION_CUSTOMER_NAME);
        \delete_option(self::$WP_OPTION_EAP_CUSTOMER);
        \delete_option(self::$WP_OPTION_MNET_SITE_REJECT);
    }

    public static function getStatus()
    {
        $customerId = self::getCustomerId();
        if (!empty($customerId)) {
            $accessToken = MnetAuthManager::getAccessToken();
            $url = MNET_API_ENDPOINT . "status" . '?access_token=' . $accessToken;
            $response = \wp_remote_get($url, DefaultOptions::$MNET_API_DEFAULT_ARGS);
            MnetAuthManager::clearExpiredToken($response);
            if (!\is_wp_error($response) && !empty($response['body']) && (!empty($response['response']) && !empty($response['response']['code']) && $response['response']['code'] === 200)) {
                $body = json_decode($response['body'], true);
                MnetPluginUtils::setWpOption(self::$WP_OPTION_EAP_CUSTOMER, !isset($body['eap']) ? 1 : intval($body['eap']));
                MnetPluginUtils::setWpOption(self::$WP_OPTION_MNET_SITE_REJECT, !isset($body['rejected']) ? 0 : intval($body['rejected']));
                MnetPluginUtils::setWpOption(self::$WP_OPTION_MNET_SITE_MAPPED, !isset($body['siteMapped']) ? 0 : intval($body['siteMapped']));
                MnetPluginUtils::setWpOption(self::$WP_OPTION_CUSTOMER_INACTIVE, !isset($body['inactive']) ? 0 : intval($body['inactive']));
                MnetPluginUtils::setWpOption(self::$WP_OPTION_MNET_SITE_STATUS, !isset($body['siteStatus']) ? '' : $body['siteStatus']);

                if (\Arr::get($body, 'adtags') !== null) {
                    MnetAdTag::populateAdtags(\Arr::get($body, 'adtags'), \Arr::get($body, 'adHeadCodes'));
                }
            }
        }
    }

    public static function logout()
    {
        self::clearLoggedInUserOptions();
        self::apiLogout();
    }

    public static function apiLogout()
    {
        $token = self::getAccessToken();
        $url = MNET_API_ENDPOINT . "logout";
        $payload = array_merge(array(
            'token' => $token,
            'domain' => MnetPluginUtils::getDomain(),
        ), MnetPluginUtils::getClientInfo());
        \wp_remote_post(
            $url,
            array_merge(
                DefaultOptions::$MNET_API_DEFAULT_ARGS,
                array(
                    'method' => 'POST',
                    'body' => $payload,
                )
            )
        );
    }
}
