<?php

namespace Mnet\Admin;

use DateTime;
use Mnet\Admin\MnetAuthManager;
use Mnet\Utils\MnetAdSlot;
use Mnet\Utils\DefaultOptions;

class MnetReportManager
{
    public static function getReport()
    {
        MnetAuthManager::returnIfSessionExpired();
        $type = $_POST['type'];
        if (isset($_POST['range']) && $_POST['range'] === "Last 7 Audited Days") {
            $lastAuditedDate = self::getLastAuditedDate();
            $from = (new DateTime($lastAuditedDate))->modify("-6 days")->format(DateTime::ISO8601);
            $to = (new DateTime($lastAuditedDate))->format(DateTime::ISO8601);
        } else {
            $from = $_POST['from'];
            $to = $_POST['to'];
        }
        $pageSize = isset($_POST['pageSize']) && $_POST['pageSize'] > 0 ? $_POST['pageSize'] : 50;
        $page = isset($_POST['page']) && $_POST['page'] > 0 ? $_POST['page'] : 1;

        $query = array(
            'access_token' => MnetAuthManager::getAccessToken(),
            'report_type' => $type,
            'from_date' => self::date($from),
            'to_date' => self::date($to),
            'page_size' => $pageSize,
            'page_number' => $page,
            'get_audit_date' => true,
        );
        $report = self::fetch($query);
        \wp_send_json($report, 200);
    }

    public static function getDashboardStats()
    {
        MnetAuthManager::returnIfSessionExpired();
        $noOfSlotsConfigured = MnetAdSlot::count();
        $revenueQuery = array(
            'report_type' => 'monthwise',
            'from_date' => self::date('first day of last month'),
            'to_date' => self::date('yesterday'),
            'access_token' => MnetAuthManager::getAccessToken(),
            'get_audit_date' => true
        );

        $lastTwoMonthsData = self::fetch($revenueQuery)['statsData']['reportItem'];
        if (empty($lastTwoMonthsData)) {
            \wp_send_json(array('noOfSlots' => $noOfSlotsConfigured, 'stats' => array()), 200);
        }

        if (count($lastTwoMonthsData) == 2 && isset($lastTwoMonthsData[0])) {
            usort($lastTwoMonthsData, function ($a, $b) {
                $date1 = new DateTime($a['date']);
                $date2 = new DateTime($b['date']);

                return $date1 == $date2 ? 0 : ($date1 > $date2 ? 1 : -1);
            });

            \wp_send_json(array('stats' => $lastTwoMonthsData, 'noOfSlots' => $noOfSlotsConfigured), 200);
        }

        $date = isset($lastTwoMonthsData[0]) ? $lastTwoMonthsData[0]['date'] : null;
        $time = new DateTime();
        $thisMonth = $time->format('F');
        if (strpos($date, $thisMonth) !== false) {
            $result = array((object) array(), (object) $lastTwoMonthsData);
        } else {
            $result = array((object) $lastTwoMonthsData, (object) array());
        }

        \wp_send_json(array('stats' => $result, 'noOfSlots' => $noOfSlotsConfigured), 200);
    }

    public static function getDashboardRevenueDetails()
    {
        MnetAuthManager::returnIfSessionExpired();
        $revenueQuery = array(
            'report_type' => 'monthwise',
            'from_date' => self::date('first day of january'),
            'to_date' => self::date('yesterday'),
            'access_token' => MnetAuthManager::getAccessToken(),
            'get_audit_date' => true
        );

        $data = self::fetch($revenueQuery);
        $ytd = $data['grandTotal']['estimatedRevenue'];
        $monthsData = $data['statsData']['reportItem'];

        $lastTwoMonthsData = array(null, null);
        $monthsData = $monthsData && count($monthsData) && !isset($monthsData[0]) ? array($monthsData) : (empty($monthsData) ? array() : $monthsData);
        array_map(function ($monthData) use (&$lastTwoMonthsData) {
            if (stripos($monthData['date'], date('F')) !== false) {
                $lastTwoMonthsData[0] = $monthData['estimatedRevenue'];
            } elseif (stripos($monthData['date'], date('F', strtotime("-1 month"))) !== false) {
                $lastTwoMonthsData[1] = $monthData['estimatedRevenue'];
            }
        }, $monthsData);

        \wp_send_json(array(
            'ytd' => $ytd,
            'monthly' => $lastTwoMonthsData,
            'lastAuditDate' => date("Y-m-d", strtotime(!empty($data['lastAuditedDate']) ? $data['lastAuditedDate'] : "-1 day"))
        ), 200);
    }

    private static function date($str = null)
    {
        $time = new DateTime($str);
        return $time->format('m/d/Y');
    }

    private static function fetch($query)
    {
        $response = \wp_remote_get(
            MNET_API_ENDPOINT . 'report?' . http_build_query(array_merge($query, array('type' => 'json'))),
            array_merge(DefaultOptions::$MNET_API_DEFAULT_ARGS, array('timeout' => 120))
        );
        MnetAuthManager::handleAccessTokenExpired($response);
        if (\is_wp_error($response) || $response['response']['code'] !== 200) {
            // $responseBody = !\is_wp_error($response) ? json_decode($response['body'], true) : null;
            \wp_send_json_error(array('error' => array('message' => "Failed to fetch reports. Click <a href='https://pubconsole.media.net/dashboard' target='_blank'> here </a> to login to media.net dashboard.")), 501);
        }
        return json_decode($response['body'], true);
    }

    private static function getLastAuditedDate()
    {
        $access_token = MnetAuthManager::getAccessToken();

        $url = MNET_API_ENDPOINT . 'last-audited-date?access_token=' . $access_token;
        $response = \wp_remote_get($url, DefaultOptions::$MNET_API_DEFAULT_ARGS);

        return \Arr::get($response, 'body');
    }
}
