<?php

namespace Mnet\Utils;

use WP_Query;
use Mnet\Admin\MnetAuthManager;


class MnetAdUtils
{
    public static function getPageType()
    {
        if (\is_home() || \is_front_page()) {
            return MNET_PAGETYPE_HOME;
        } elseif (\is_search()) {
            return MNET_PAGETYPE_SEARCH;
        } elseif (\is_single()) {
            return MNET_PAGETYPE_ARTICLE;
        } elseif (\is_category()) {
            return MNET_PAGETYPE_CATEGORY;
        } elseif (\is_archive() || (\is_home() && !\is_front_page())) {
            return MNET_PAGETYPE_ARCHIVE;
        } elseif (\is_page()) {
            return MNET_PAGETYPE_STATIC;
        } elseif (\is_admin()) {
            return MNET_PAGETYPE_ADMIN;
        } else {
            return MNET_PAGETYPE_ALL;
        }
    }

    public static function trimUrl($url)
    {
        $url = preg_replace('/http(s)?\:\/\//', '', $url);
        $url = preg_replace('/\/.*\.php\//', '/', $url);
        if ($url[strlen($url) - 1] == '/') {
            $url = substr($url, 0, strlen($url) - 1);
        }
        return $url;
    }

    public static function getUrls()
    {
        MnetAuthManager::returnIfSessionExpired();
        global $wpdb;
        $permalink = array();
        $permalink[MNET_PAGETYPE_HOME] = array(array('name' => 'Home', 'url' => self::trimUrl(\get_home_url())));
        // Urls of all posts
        $posts = new WP_Query('post_type=any&posts_per_page=-1&post_status=publish');
        $posts = $posts->posts;
        $permalink[MNET_PAGETYPE_ARTICLE] = array();
        $permalink[MNET_PAGETYPE_STATIC] = array();
        foreach ($posts as $post) {
            switch ($post->post_type) {
                case 'revision':
                case 'nav_menu_item':
                    $link = '';
                    break;
                case 'page':
                    array_push($permalink[MNET_PAGETYPE_STATIC], array('name' => $post->post_title, 'url' => self::trimUrl(\get_page_link($post->ID))));
                    $link = '';
                    break;
                case 'post':
                    $link = self::trimUrl(\get_permalink($post->ID));
                    break;
                case 'attachment':
                    $link = self::trimUrl(\get_attachment_link($post->ID));
                    break;
                default:
                    $link = self::trimUrl(\get_post_permalink($post->ID));
                    break;
            }
            if (strlen($link)) {
                array_push($permalink[MNET_PAGETYPE_ARTICLE], array('name' => $post->post_title, 'url' => $link));
            }
        }

        // Urls of categories
        $categories = \get_categories();
        $permalink[MNET_PAGETYPE_CATEGORY] = array();
        foreach ($categories as $category) {
            array_push($permalink[MNET_PAGETYPE_CATEGORY], array('name' => $category->name, 'url' => self::trimUrl(\get_category_link($category->term_id))));
        }

        // Urls of archive pages
        $permalink[MNET_PAGETYPE_ARCHIVE] = array();
        $post_dates = $wpdb->get_results("SELECT DISTINCT MONTH( post_date ) AS month ,	YEAR( post_date ) AS year FROM $wpdb->posts WHERE post_status = 'publish' and post_date <= now( ) and post_type = 'post' GROUP BY month , year ORDER BY post_date DESC");
        foreach ($post_dates as $date) {
            $monthName = date('F', mktime(0, 0, 0, $date->month, 10));
            array_push($permalink[MNET_PAGETYPE_ARCHIVE], array('name' => $monthName . ' ' . $date->year, 'url' => self::trimUrl(\home_url()) . '/' . $date->year . '/' . str_pad($date->month, 2, '0', STR_PAD_LEFT)));
        }

        return $permalink;
    }

    public static function getCssOptions($options)
    {
        $default_css_options = include(__DIR__ . '/defaultCssOptions.php');
        return array_merge($default_css_options, $options);
    }
}
